//===--------------- P3_Fer.h - Clique substitution of P_3 ----------------===//
//
//              Register Allocation via Coloring of Chordal Graphs 
//
// This class is an interface for clique substitutions of P3
//
// Date: Oct 7th, 2007
//
//===----------------------------------------------------------------------===//
//
// Clique substitutions of P_3 are graphs that can be divided into three sets
// of nodes: X, Y and Z. Nodes in X, Y and Z form cliques. Also, nodes of X and
// Y are pairwise adjance. The same is true for nodes of Y and Z. However,
// nodes of X and Z are not adjacent. Thus, any of these graphs has two maximal
// cliques: X union Y and Y union Z.
// Notice that this class has only been implemented for the X86 architecture.
//
//===----------------------------------------------------------------------===//


#ifndef FACADE_FER_H
#define FACADE_FER_H

namespace llvm {

  class MachineInstr;

  class FacadeP3_Fer {

    public:
    /// The virtual dtor is only to conform to C++ standards. This class is
    /// not meant to be instantianted:
    virtual ~FacadeP3_Fer() {}

    /// Add 'reg' as a node of component X
    virtual void addX(unsigned reg) {}

    /// Add 'reg' as a node of component Y
    virtual void addY(unsigned reg) {}

    /// Add 'reg' as a node of component Z
    virtual void addZ(unsigned reg) {}

    /// The number of virtuals in this partition:
    virtual unsigned size() const { return 0; }

    // This method prints the location of each variable that is alive at the
    // beginning of the program point that this graph represents.
    virtual void print_in_set() const {}

    /// This method colors this clique substitution of P_3 with the free
    /// registers of this object. The array preferences associates virtual
    /// variables to colors. This vector determines a color assignment that
    /// would reduce the number of copies in the final assembly code. Given
    /// two successive KP3 (Clique substitution of P_3), q1 and q2, the
    /// color assignment in q1 is used to build the preference vector that will
    /// be passed to q2.
    virtual MachineInstr * color() { return NULL; }

    // This method return true if the graph has nodes of type short in any of
    // its maximal cliques.
    virtual bool contains_short() const { return false; }
   
    // This method return true if the graph has nodes of type long in any of its
    // maximal cliques.
    virtual bool contains_long() const { return false; }

    // This method returns true if the contents of p_reg can be evicted to
    // memory. The only situations when this cannot happen is if p_reg is
    // precolored, or if p_reg is holding a virtual that is used in the current
    // instruction.
    virtual bool can_spill(unsigned p_reg) const { return false; }

    // Number of times the puzzle solver had to iterate due to spilling.
    virtual unsigned get_num_iterations() const { return 0; }

    /// Prints the assignment of physical registers to virtuals that exist
    /// in the X component.
    virtual void dump_X_colors() const {}

    /// Prints the assignment of physical registers to virtuals in the Z
    /// component, that is, after the instruction that this object surrounds.
    virtual void dump_Z_colors() const {}
  };

}

#endif
