//===----------------- Line_Fer.h - Phi Equivalence Class -----------------===//
//
//              Register Allocation via Coloring of Chordal Graphs 
//
// This class represents one continuous live range.
//
// Date: Jun 30th, 2006
//
//===----------------------------------------------------------------------===//
//
// This is the live range of a variable. Actually, the live range of a variable
// is formed by the union of many of these objects. Each element in this union
// represents a contigouous interval in which the variable is alive.
//
//===----------------------------------------------------------------------===//

#ifndef LINE_FER_H
#define LINE_FER_H

#include <iostream>

namespace llvm {

  class Line_Fer {

    public:
    unsigned start;
    unsigned end;
    unsigned reg;

    /// Constructor method.
    Line_Fer(unsigned s, unsigned e, unsigned r) : start(s), end(e), reg(r) {
    }

    /// contains - Return true if the index is covered by this range.
    bool contains(unsigned I) const {
      return start <= I && I < end;
    }

    /// Returns true if this interval starts after 'L'.
    bool operator>(const Line_Fer & L) const {
      return L.start < this->start;
    }

    /// Returns true if this interval starts before 'L'.
    bool operator<(const Line_Fer & L) const {
      return this->start < L.start;
    }

    /// true if "L.contains(start) || this->contains(L.start)"
    bool overlap(const Line_Fer & L) const {
      return L.contains(start) || this->contains(L.start);
    }

    private:
  };

  inline std::ostream & operator<<(std::ostream & os, const Line_Fer & lf) {
    os << "[" << lf.start << ", " << lf.end << "]: (" << lf.reg << ")";
    return os;
  }

  class LineCompareEnd {
    public :
      bool operator()(const Line_Fer * x, const Line_Fer * y) const {
        if(x->end != y->end) {
            return x->end < y->end;
        } else {
            return x->reg < y->reg;
        }
      }
  };

}

#endif
